/*
 * This file is part of GtkJson.
 *
 * Copyright © 2023 Canek Peláez Valdés
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

namespace GtkJson.Application {

    /**
     * Class for the GtkJson application window.
     */
    [GtkTemplate (ui = "/mx/unam/GtkJson/application-window.ui")]
    public class ApplicationWindow : Gtk.ApplicationWindow {

        [GtkChild]
        private unowned Gtk.Label subtitle;

        [GtkChild]
        private unowned GLib.ListStore nodes_model;

        [GtkChild]
        private unowned Gtk.SingleSelection nodes_selection;

        /* The tree model. */
        private Gtk.TreeListModel tree_model;

        /**
         * Constructs the application window.
         * @param application the application.
         */
        public ApplicationWindow(Application application) {
            GLib.Object(application: application);
            tree_model = new Gtk.TreeListModel(nodes_model, false, false,
                                               create_row);
            nodes_selection.model = tree_model;
        }

        private GLib.ListModel? create_row(GLib.Object item) {
            if (item == null || !(item is NodeRow))
                return null;
            var row = item as NodeRow;
            switch (row.node.get_node_type()) {
            case Json.NodeType.OBJECT: return create_object_row(row.node);
            case Json.NodeType.ARRAY:  return create_array_row(row.node);
            default:                   return null;
            }
        }

        private GLib.ListModel? create_object_row(Json.Node node) {
            var object = node.get_object();
            var model = new GLib.ListStore(typeof(NodeRow));
            GLib.List<weak string> m = object.get_members();
            while (m != null) {
                var n = object.get_member(m.data);
                var row = new NodeRow(n, m.data);
                model.append(row);
                m = (owned)m.next;
            }
            return model;
        }

        private GLib.ListModel? create_array_row(Json.Node node) {
            var array = node.get_array();
            var model = new GLib.ListStore(typeof(NodeRow));
            for (uint i = 0; i < array.get_length(); i++) {
                var n = array.get_element(i);
                var row = new NodeRow(n, @"$(i)");
                model.append(row);
            }
            return model;
        }

        public void add_root(Json.Node node) {
            var row = new NodeRow(node, "ROOT");
            nodes_model.append(row);
        }

        public void set_subtitle(string subtitle) {
            this.subtitle.label = subtitle;
        }

        /* The on close request callback. */
        [GtkCallback]
        private bool on_close_request(Gtk.Window window) {
            var app = application as Application;
            app.activate_action("app.quit", null);
            return false;
        }
    }
}

