/*
 * This file is part of GtkJson.
 *
 * Copyright © YEAR(S) Canek Peláez Valdés canek@ciencias.unam.mx
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * Namespace for the GtkJson Gtk application.
 */
namespace GtkJson.Application {

    /**
     * Class for the GtkJson application.
     */
    public class Application : Gtk.Application {

        /* CSS resource. */
        private const string CSS_RESOURCE =
            "/mx/unam/GtkJson/style.css";

        /* The application window. */
        public ApplicationWindow application_window;

        /**
         * Starts up the application.
         */
        public override void startup() {
            base.startup();
            if (application_window == null)
                application_window = new ApplicationWindow(this);

            var open_action = new GLib.SimpleAction("open", null);
            add_action(open_action);
            string[] open_accels = { "<Ctrl>O" };
            set_accels_for_action("app.open", open_accels);
            open_action.activate.connect(open_handler);

            var about_action = new GLib.SimpleAction("about", null);
            add_action(about_action);
            string[] about_accels = { "<Ctrl>A" };
            set_accels_for_action("app.about", about_accels);
            about_action.activate.connect(about_handler);

            var quit_action = new GLib.SimpleAction("quit", null);
            add_action(quit_action);
            string[] quit_accels = { "<Ctrl>Q" };
            set_accels_for_action("app.quit", quit_accels);
            quit_action.activate.connect(quit_handler);
        }

        /**
         * Activates the application.
         */
        public override void activate() {
            Gdk.Display display = application_window.get_display();
            var provider = new Gtk.CssProvider();
            provider.load_from_resource(CSS_RESOURCE);
            Gtk.StyleContext.add_provider_for_display(
                display, provider, Gtk.STYLE_PROVIDER_PRIORITY_USER);
            application_window.present();
        }

        /* Open a JSON file. */
        private void open_handler() {
            var dialog = new Gtk.FileDialog();
            dialog.accept_label = "Open JSON file";
            var filters = new GLib.ListStore(typeof(Gtk.FileFilter));
            var filter = new Gtk.FileFilter();
            filter.add_mime_type("application/json");
            filters.append(filter);
            dialog.filters = filters;
            dialog.open.begin(application_window, null, open_result);
        }

        private void open_result(GLib.Object? object,
                                 GLib.AsyncResult result) {
            var dialog = object as Gtk.FileDialog;
            if (dialog == null)
                return;
            try {
                var file = dialog.open.end(result);
                if (file != null)
                    load(file);
            } catch (Gtk.DialogError error) {
                return;
            } catch (GLib.Error error) {
                GLib.error(@"Error: $(error.message)");
            }
        }

        private void load(GLib.File file) {
            try {
                var parser = new Json.Parser();
                parser.load_from_file(file.get_path());
                var node = parser.get_root();
                application_window.add_root(node);
                application_window.set_subtitle(file.get_basename());
            } catch (GLib.Error error) {
                stderr.printf("Identification: %" + uint32.FORMAT + "\n", error.domain);
                GLib.error("Error: $(error.message)");
            }
        }

        /* Shows the about dialog. */
        private void about_handler() {
            Gtk.show_about_dialog(
                application_window,
                "authors", new string[] { "Canek Peláez Valdés canek@ciencias.unam.mx" },
                "comments", "A simple JSON viewer",
                "copyright", "Copyright © 2023 Canek Peláez Valdés",
                "license-type", Gtk.License.GPL_3_0,
                "website", "https://gtkjson.org/GtkJson",
                "website-label", "Project homepage",
                "wrap-license", true);
        }

        /* Quits the application. */
        private void quit_handler() {
            quit();
        }
    }
}
